@@ -1,18 +1,12 @@
 /*
- * Copyright 2003-2005 Gentoo Foundation
+ * Copyright 2003-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.30 2005/12/07 01:04:05 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.34 2006/01/10 01:40:15 vapier Exp $
  *
- * Copyright 2005 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-#include <stdio.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
 #include "paxinc.h"
 
 #define argv0 "paxelf"
@@ -332,7 +326,7 @@ const char *get_elfstttype(int type)
 	((buff[EI_CLASS] == ELFCLASS32 || buff[EI_CLASS] == ELFCLASS64) && \
 	 (buff[EI_DATA] == ELFDATA2LSB || buff[EI_DATA] == ELFDATA2MSB) && \
 	 (buff[EI_VERSION] == EV_CURRENT))
-elfobj *readelf(const char *filename)
+elfobj *_readelf(const char *filename, int read_only)
 {
 	struct stat st;
 	int fd;
@@ -341,7 +335,7 @@ elfobj *readelf(const char *filename)
 	if (stat(filename, &st) == -1)
 		return NULL;
 
-	if ((fd = open(filename, O_RDONLY)) == -1)
+	if ((fd = open(filename, (read_only ? O_RDONLY : O_RDWR))) == -1)
 		return NULL;
 
 	/* make sure we have enough bytes to scan e_ident */
@@ -355,7 +349,7 @@ elfobj *readelf(const char *filename)
 
 	elf->fd = fd;
 	elf->len = st.st_size;
-	elf->data = (char*)mmap(0, elf->len, PROT_READ, MAP_PRIVATE, fd, 0);
+	elf->data = (char*)mmap(0, elf->len, PROT_READ | (read_only ? 0 : PROT_WRITE), (read_only ? MAP_PRIVATE : MAP_SHARED), fd, 0);
 	if (elf->data == (char*)MAP_FAILED) {
 		warn("mmap on '%s' of %li bytes failed :(", filename, (unsigned long)elf->len);
 		goto free_elf_and_return;
@@ -428,6 +422,7 @@ close_fd_and_return:
 	close(fd);
 	return NULL;
 }
+elfobj *readelf(const char *filename) { return _readelf(filename, 1); }
 
 /* undo the readelf() stuff */
 void unreadelf(elfobj *elf)
@@ -452,40 +447,33 @@ char *pax_short_hf_flags(unsigned long flags)
 	return buffer;
 }
 
-/* the display logic is:
+/* PT_PAX_FLAGS are tristate ...
+ * the display logic is:
  * lower case: explicitly disabled
  * upper case: explicitly enabled
- * - : default */
+ *      -    : default */
 char *pax_short_pf_flags(unsigned long flags)
 {
 	static char buffer[7];
 
-	/* PT_PAX_FLAGS are tristate */
-	buffer[0] = (flags & PF_PAGEEXEC ? 'P' : '-');
-	buffer[0] = (flags & PF_NOPAGEEXEC ? 'p' : buffer[0]);
-
-	buffer[1] = (flags & PF_SEGMEXEC ? 'S' : '-');
-	buffer[1] = (flags & PF_NOSEGMEXEC ? 's' : buffer[1]);
-
-	buffer[2] = (flags & PF_MPROTECT ? 'M' : '-');
-	buffer[2] = (flags & PF_NOMPROTECT ? 'm' : buffer[2]);
-
-	buffer[3] = (flags & PF_RANDEXEC ? 'X' : '-');
-	buffer[3] = (flags & PF_NORANDEXEC ? 'x' : buffer[3]);
-
-	buffer[4] = (flags & PF_EMUTRAMP ? 'E' : '-');
-	buffer[4] = (flags & PF_NOEMUTRAMP ? 'e' : buffer[4]);
-
-	buffer[5] = (flags & PF_RANDMMAP ? 'R' : '-');
-	buffer[5] = (flags & PF_NORANDMMAP ? 'r' : buffer[5]);
+#define PAX_STATE(pf_on, pf_off, disp_on, disp_off) \
+	(flags & pf_on ? disp_on : (flags & pf_off ? disp_off : '-'))
 
+	buffer[0] = PAX_STATE(PF_PAGEEXEC, PF_NOPAGEEXEC, 'P', 'p');
+	buffer[1] = PAX_STATE(PF_SEGMEXEC, PF_NOSEGMEXEC, 'S', 's');
+	buffer[2] = PAX_STATE(PF_MPROTECT, PF_NOMPROTECT, 'M', 'm');
+	buffer[3] = PAX_STATE(PF_RANDEXEC, PF_NORANDEXEC, 'X', 'x');
+	buffer[4] = PAX_STATE(PF_EMUTRAMP, PF_NOEMUTRAMP, 'E', 'e');
+	buffer[5] = PAX_STATE(PF_RANDMMAP, PF_NORANDMMAP, 'R', 'r');
 	buffer[6] = 0;
 
-  
-	if (((flags & PF_PAGEEXEC) && (flags & PF_NOPAGEEXEC)) || ((flags & PF_SEGMEXEC) && (flags & PF_NOSEGMEXEC))
-		|| ((flags & PF_RANDMMAP) && (flags & PF_NORANDMMAP)) || ((flags & PF_RANDEXEC) && (flags & PF_NORANDEXEC))
-		|| ((flags & PF_EMUTRAMP) && (flags &  PF_NOEMUTRAMP)) || ((flags & PF_RANDMMAP) && (flags & PF_NORANDMMAP)))
-		warn("inconsistent state detected. flags=%lu\n", flags);
+	if (((flags & PF_PAGEEXEC) && (flags & PF_NOPAGEEXEC)) || \
+	    ((flags & PF_SEGMEXEC) && (flags & PF_NOSEGMEXEC)) || \
+	    ((flags & PF_RANDMMAP) && (flags & PF_NORANDMMAP)) || \
+	    ((flags & PF_RANDEXEC) && (flags & PF_NORANDEXEC)) || \
+	    ((flags & PF_EMUTRAMP) && (flags & PF_NOEMUTRAMP)) || \
+	    ((flags & PF_RANDMMAP) && (flags & PF_NORANDMMAP)))
+		warn("inconsistent state detected.  flags=%lX\n", flags);
 
 	return buffer;
 }
@@ -535,3 +523,19 @@ void *elf_findsecbyname(elfobj *elf, const char *name)
 
 	return ret;
 }
+
+int elf_max_pt_load(elfobj *elf)
+{
+#define MAX_PT_LOAD(B) \
+	if (elf->elf_class == ELFCLASS ## B) { \
+	Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
+	switch (EGET(ehdr->e_ident[EI_OSABI])) { \
+	case ELFOSABI_NONE: \
+	case ELFOSABI_ARM:     return 2; \
+	case ELFOSABI_OPENBSD: return 7; \
+	} }
+	MAX_PT_LOAD(32)
+	MAX_PT_LOAD(64)
+
+	return 0;
+}
