@@ -1,10 +1,10 @@
 /*
- * Copyright 2005 Gentoo Foundation
+ * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/porting.h,v 1.7 2005/12/29 12:56:41 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/porting.h,v 1.10 2006/01/10 01:31:17 vapier Exp $
  *
- * Copyright 2005 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
  *
  * Make sure all of the common elf stuff is setup as we expect
  */
@@ -12,6 +12,23 @@
 #ifndef _PORTING_H
 #define _PORTING_H
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <errno.h>
+#include <string.h>
+#include <unistd.h>
+#include <inttypes.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <dirent.h>
+#include <limits.h>
+#include <fcntl.h>
+#include <assert.h>
+#include <getopt.h>
+#include <libgen.h>
+#include <ctype.h>
+#include <pwd.h>
+
 #include <sys/mman.h>
 #include "elf.h"
 #if defined(__linux__)
@@ -21,8 +38,8 @@
 # include <sys/endian.h>
 #endif
 
-#if defined(__APPLE__) || defined(__NetBSD__)
-# define SET_STDOUT(fp) err("Darwin/NetBSD has stupid stdout handling")
+#if defined(__APPLE__) || defined(__NetBSD__) || defined(__OpenBSD__)
+# define SET_STDOUT(fp) err("Darwin/NetBSD/OpenBSD have stupid stdout handling")
 #else
 # define SET_STDOUT(fp) stdout = fp
 #endif
