@@ -1,29 +1,15 @@
 /*
- * Copyright 2003-2005 Gentoo Foundation
+ * Copyright 2003-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.97 2005/12/29 14:03:25 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.103 2006/01/11 23:46:21 vapier Exp $
  *
- * Copyright 2003-2005 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2004-2005 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2003-2006 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2004-2006 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <libgen.h>
-#include <limits.h>
-#include <string.h>
-#include <errno.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
-#include <fcntl.h>
-#include <dirent.h>
-#include <getopt.h>
-#include <assert.h>
 #include "paxinc.h"
 
-static const char *rcsid = "$Id: scanelf.c,v 1.97 2005/12/29 14:03:25 vapier Exp $";
+static const char *rcsid = "$Id: scanelf.c,v 1.103 2006/01/11 23:46:21 vapier Exp $";
 #define argv0 "scanelf"
 
 #define IS_MODIFIER(c) (c == '%' || c == '#')
@@ -67,8 +53,9 @@ static char *find_sym = NULL, *versioned_symname = NULL;
 static char *find_lib = NULL;
 static char *out_format = NULL;
 static char *search_path = NULL;
+static char fix_elf = 0;
 static char gmatch = 0;
-static char printcache = 0;
+static char use_ldcache = 0;
 
 
 caddr_t ldcache = 0;
@@ -152,8 +139,8 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 {
 	static char ret[12];
 	char *found;
-	unsigned long i, shown;
-	unsigned char multi_stack, multi_relro, multi_load;
+	unsigned long i, shown, multi_stack, multi_relro, multi_load;
+	int max_pt_load;
 
 	if (!show_phdr) return NULL;
 
@@ -161,6 +148,7 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 
 	shown = 0;
 	multi_stack = multi_relro = multi_load = 0;
+	max_pt_load = elf_max_pt_load(elf);
 
 #define SHOW_PHDR(B) \
 	if (elf->elf_class == ELFCLASS ## B) { \
@@ -181,7 +169,7 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 				offset = 4; \
 				check_flags = PF_X; \
 			} else if (EGET(phdr[i].p_type) == PT_LOAD) { \
-				if (multi_load++ > 2) warnf("%s: more than 2 PT_LOAD's !?", elf->filename); \
+				if (multi_load++ > max_pt_load) warnf("%s: more than %i PT_LOAD's !?", elf->filename, max_pt_load); \
 				found = found_load; \
 				offset = 8; \
 				check_flags = PF_W|PF_X; \
@@ -190,6 +178,11 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 			flags = EGET(phdr[i].p_flags); \
 			if (be_quiet && ((flags & check_flags) != check_flags)) \
 				continue; \
+			if (fix_elf && ((flags & PF_X) != flags)) { \
+				ESET(phdr[i].p_flags, flags & (PF_X ^ (size_t)-1)); \
+				ret[3] = ret[7] = '!'; \
+				flags = EGET(phdr[i].p_flags); \
+			} \
 			memcpy(ret+offset, gnu_short_stack_flags(flags), 3); \
 			*found = 1; \
 			++shown; \
@@ -370,25 +363,27 @@ static char *scanelf_file_textrels(elfobj *elf, char *found_textrels, char *foun
 	return NULL;
 }
 
-static void rpath_security_checks(elfobj *, char *);
-static void rpath_security_checks(elfobj *elf, char *item) {
+static void rpath_security_checks(elfobj *, char *, const char *);
+static void rpath_security_checks(elfobj *elf, char *item, const char *dt_type)
+{
 	struct stat st;
 	switch (*item) {
 		case '/': break;
 		case '.':
-			warnf("Security problem with relative RPATH '%s' in %s", item, elf->filename);
+			warnf("Security problem with relative %s '%s' in %s", dt_type, item, elf->filename);
 			break;
+		case ':':
 		case '\0':
-			warnf("Security problem NULL RPATH in %s", elf->filename);
+			warnf("Security problem NULL %s in %s", dt_type, elf->filename);
 			break;
 		case '$':
 			if (fstat(elf->fd, &st) != -1)
 				if ((st.st_mode & S_ISUID) || (st.st_mode & S_ISGID))
-					warnf("Security problem with RPATH='%s' in %s with mode set of %o", 
-						item, elf->filename, st.st_mode & 07777);
+					warnf("Security problem with %s='%s' in %s with mode set of %o", 
+					      dt_type, item, elf->filename, st.st_mode & 07777);
 			break;
 		default:
-			warnf("Maybe? sec problem with RPATH='%s' in %s", item, elf->filename);
+			warnf("Maybe? sec problem with %s='%s' in %s", dt_type, item, elf->filename);
 			break;
 	}
 }
@@ -434,6 +429,9 @@ static void scanelf_file_rpath(elfobj *elf, char *found_rpath, char **ret, size_
 				if (offset < (Elf ## B ## _Off)elf->len) { \
 					if (*r) warn("ELF has multiple %s's !?", get_elfdtype(word)); \
 					*r = (char*)(elf->data + offset); \
+					/* cache the length in case we need to nuke this section later on */ \
+					if (fix_elf) \
+						offset = strlen(*r); \
 					/* If quiet, don't output paths in ld.so.conf */ \
 					if (be_quiet) { \
 						size_t len; \
@@ -443,22 +441,67 @@ static void scanelf_file_rpath(elfobj *elf, char *found_rpath, char **ret, size_
 						start = *r; \
 						/* scan each path in : delimited list */ \
 						while (start) { \
-							rpath_security_checks(elf, start); \
+							rpath_security_checks(elf, start, get_elfdtype(word)); \
 							end = strchr(start, ':'); \
 							len = (end ? abs(end - start) : strlen(start)); \
-							for (s = 0; ldpaths[s]; ++s) { \
-								if (!strncmp(ldpaths[s], start, len) && !ldpaths[s][len]) { \
-									*r = (end ? end + 1 : NULL); \
-									break; \
-								} \
-							} \
-							if (!*r || !ldpaths[s] || !end) \
-								start = NULL; \
+							if (use_ldcache) \
+								for (s = 0; ldpaths[s]; ++s) \
+									if (!strncmp(ldpaths[s], start, len) && !ldpaths[s][len]) { \
+										*r = end; \
+										/* corner case ... if RPATH reads "/usr/lib:", we want \
+										 * to show ':' rather than '' */ \
+										if (end && end[1] != '\0') \
+											(*r)++; \
+										break; \
+									} \
+							if (!*r || !end) \
+								break; \
 							else \
 								start = start + len + 1; \
 						} \
 					} \
-					if (*r) *found_rpath = 1; \
+					if (*r) { \
+						if (fix_elf > 2 || **r == '\0') { \
+							/* just nuke it */ \
+							nuke_it##B: \
+							memset(*r, 0x00, offset); \
+							*r = NULL; \
+							ESET(dyn->d_tag, DT_DEBUG); \
+							ESET(dyn->d_un.d_ptr, 0); \
+						} else if (fix_elf) { \
+							/* try to clean "bad" paths */ \
+							size_t len, tmpdir_len; \
+							char *start, *end; \
+							const char *tmpdir; \
+							start = *r; \
+							tmpdir = (getenv("TMPDIR") ? : "."); \
+							tmpdir_len = strlen(tmpdir); \
+							while (1) { \
+								end = strchr(start, ':'); \
+								if (start == end) { \
+									eat_this_path##B: \
+									len = strlen(end); \
+									memmove(start, end+1, len); \
+									start[len-1] = '\0'; \
+									end = start - 1; \
+								} else if (tmpdir && !strncmp(start, tmpdir, tmpdir_len)) { \
+									if (!end) { \
+										if (start == *r) \
+											goto nuke_it##B; \
+										*--start = '\0'; \
+									} else \
+										goto eat_this_path##B; \
+								} \
+								if (!end) \
+									break; \
+								start = end + 1; \
+							} \
+							if (**r == '\0') \
+								goto nuke_it##B; \
+						} \
+						if (*r) \
+							*found_rpath = 1; \
+					} \
 				} \
 				++dyn; \
 			} \
@@ -606,7 +649,7 @@ static const char *scanelf_file_needed_lib(elfobj *elf, char *found_needed, char
 					if (op == 0) { \
 						if (!be_wewy_wewy_quiet) { \
 							if (*found_needed) xchrcat(ret, ',', ret_len); \
-							if (printcache) \
+							if (use_ldcache) \
 								if ((p = lookup_cache_lib(elf, needed)) != NULL) \
 									needed = p; \
 							xstrcat(ret, needed, ret_len); \
@@ -835,7 +878,7 @@ static void scanelf_file(const char *filename)
 	found_sym = found_lib = found_file = found_textrels = 0;
 
 	/* verify this is real ELF */
-	if ((elf = readelf(filename)) == NULL) {
+	if ((elf = _readelf(filename, !fix_elf)) == NULL) {
 		if (be_verbose > 2) printf("%s: not an ELF\n", filename);
 		return;
 	}
@@ -1118,7 +1161,7 @@ static void scanelf_envpath()
 
 
 /* usage / invocation handling functions */
-#define PARSE_FLAGS "plRmyxetrnLibSs:gN:TaqvF:f:o:BhV"
+#define PARSE_FLAGS "plRmyXxetrnLibSs:gN:TaqvF:f:o:BhV"
 #define a_argument required_argument
 static struct option const long_opts[] = {
 	{"path",      no_argument, NULL, 'p'},
@@ -1126,6 +1169,7 @@ static struct option const long_opts[] = {
 	{"recursive", no_argument, NULL, 'R'},
 	{"mount",     no_argument, NULL, 'm'},
 	{"symlink",   no_argument, NULL, 'y'},
+	{"fix",       no_argument, NULL, 'X'},
 	{"pax",       no_argument, NULL, 'x'},
 	{"header",    no_argument, NULL, 'e'},
 	{"textrel",   no_argument, NULL, 't'},
@@ -1156,13 +1200,14 @@ static const char *opts_help[] = {
 	"Scan all directories in /etc/ld.so.conf",
 	"Scan directories recursively",
 	"Don't recursively cross mount points",
-	"Don't scan symlinks\n",
+	"Don't scan symlinks",
+	"Try and 'fix' bad things (use with -r/-e)\n",
 	"Print PaX markings",
 	"Print GNU_STACK/PT_LOAD markings",
 	"Print TEXTREL information",
 	"Print RPATH information",
 	"Print NEEDED information",
-	"Resolve NEEDED information (use with -n)",
+	"Utilize ld.so.cache information (use with -r/-n)",
 	"Print INTERP information",
 	"Print BIND information",
 	"Print SONAME information",
@@ -1231,8 +1276,8 @@ static void parseargs(int argc, char *argv[])
 			break;
 		case 'h': usage(EXIT_SUCCESS); break;
 		case 'f':
-			if (from_file) err("Don't specify -f twice");
-			from_file = xstrdup(optarg);
+			if (from_file) warn("You prob don't want to specify -f twice");
+			from_file = optarg;
 			break;
 		case 'o': {
 			FILE *fp = NULL;
@@ -1262,13 +1307,14 @@ static void parseargs(int argc, char *argv[])
 		}
 
 		case 'g': gmatch = 1; /* break; any reason we dont breal; here ? */
-		case 'L': printcache = 1; break;
+		case 'L': use_ldcache = 1; break;
 		case 'y': scan_symlink = 0; break;
 		case 'B': show_banner = 0; break;
 		case 'l': scan_ldpath = 1; break;
 		case 'p': scan_envpath = 1; break;
 		case 'R': dir_recurse = 1; break;
 		case 'm': dir_crossmount = 0; break;
+		case 'X': ++fix_elf; break;
 		case 'x': show_pax = 1; break;
 		case 'e': show_phdr = 1; break;
 		case 't': show_textrel = 1; break;
@@ -1344,13 +1390,12 @@ static void parseargs(int argc, char *argv[])
 	if (be_verbose > 2) printf("Format: %s\n", out_format);
 
 	/* now lets actually do the scanning */
-	if (scan_ldpath || (show_rpath && be_quiet))
+	if (scan_ldpath || use_ldcache)
 		load_ld_so_conf();
 	if (scan_ldpath) scanelf_ldpath();
 	if (scan_envpath) scanelf_envpath();
 	if (from_file) {
 		scanelf_from_file(from_file);
-		free(from_file);
 		from_file = *argv;
 	}
 	if (optind == argc && !scan_ldpath && !scan_envpath && !from_file)
